package cn.fw.morax.rpc.oop;

import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.StringUtils;
import cn.fw.morax.rpc.oop.dto.DealerDTO;
import cn.fw.morax.rpc.oop.dto.GroupDTO;
import cn.fw.morax.rpc.oop.dto.ShopDTO;
import cn.fw.oop.sdk.DealerApiService;
import cn.fw.oop.sdk.GroupApiService;
import cn.fw.oop.sdk.ShopApiService;
import cn.fw.oop.sdk.result.DealerInfoVo;
import cn.fw.oop.sdk.result.GroupInfoVo;
import cn.fw.oop.sdk.result.ShopInfoVo;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.BoundValueOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * OOP系统
 * create at 2019-03-07
 *
 * @author kurisu
 */
@Slf4j
@Service
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class OopRpcService {
    private final ShopApiService shopApiService;
    private final DealerApiService dealerApiService;
    private final GroupApiService groupApiService;
    private final StringRedisTemplate stringRedisTemplate;

    @Value("${spring.cache.custom.global-prefix}:rpc:oop")
    @Getter
    private String keyPrefix;


    /**
     * 根据ID获取门店信息
     *
     * @param shopId 门店ID
     * @return 门店信息
     */
    public ShopDTO queryShop(final Long shopId) {
        if (shopId == null) {
            return null;
        }
        final String key = generateKey(OopTypeEnum.SHOP, shopId);
        BoundValueOperations<String, String> valueOps = stringRedisTemplate.boundValueOps(key);
        String json = valueOps.get();
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, ShopDTO.class);
        }
        Message<ShopInfoVo> msg = shopApiService.getShopInfo(shopId);
        log.info("shopApiService.getShopInfo 返回结果: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
        if (msg.isSuccess() && null != msg.getData()) {
            final ShopDTO shop = new ShopDTO();
            BeanUtils.copyProperties(msg.getData(), shop);
            valueOps.setIfAbsent(JSONObject.toJSONString(shop), 12L, TimeUnit.HOURS);
            return shop;
        }
        return null;
    }


    /**
     * 根据ID获取商家信息
     *
     * @param dealerId 商家ID
     * @return 商家信息
     */
    public DealerDTO queryDealer(final Long dealerId) {
        if (dealerId == null) {
            return null;
        }
        String key = generateKey(OopTypeEnum.DEALER, dealerId);
        BoundValueOperations<String, String> valueOps = stringRedisTemplate.boundValueOps(key);
        String json = valueOps.get();
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, DealerDTO.class);
        }
        Message<DealerInfoVo> msg = dealerApiService.getDealerInfo(dealerId);
        log.info("dealerApiService.getDealerInfo: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
        if (msg.isSuccess() && null != msg.getData()) {
            final DealerDTO dealer = new DealerDTO();
            BeanUtils.copyProperties(msg.getData(), dealer);
            valueOps.setIfAbsent(JSONObject.toJSONString(dealer), 12L, TimeUnit.HOURS);
            return dealer;
        }
        return null;
    }


    /**
     * 查询集团信息
     *
     * @param groupId 集团id
     * @return
     */
    public GroupDTO queryGroup(final Long groupId) {
        if (groupId == null) {
            return null;
        }
        String key = generateKey(OopTypeEnum.GROUP, groupId);
        BoundValueOperations<String, String> valueOps = stringRedisTemplate.boundValueOps(key);
        String json = valueOps.get();
        if (StringUtils.isValid(json)) {
            return JSONObject.parseObject(json, GroupDTO.class);
        }
        log.info("调用OOP系统查询集团信息, groupId: {}", groupId);
        final Message<GroupInfoVo> msg = groupApiService.getGroupInfo(groupId);
        if (msg.isSuccess() && msg.getData() != null) {
            log.info("调用OOP系统查询集团信息成功, result: {}", msg.getData());
            GroupDTO groupDTO = new GroupDTO();
            BeanUtils.copyProperties(msg.getData(), groupDTO);
            valueOps.setIfAbsent(JSONObject.toJSONString(groupDTO), 12L, TimeUnit.HOURS);
            return groupDTO;
        }
        return null;
    }

    /**
     * 查询全部集团信息
     *
     * @return
     */
    public List<GroupDTO> allGroups() {
        String key = String.format("%s:%s:%s", getKeyPrefix(), "all", OopTypeEnum.GROUP.name());
        BoundListOperations<String, String> boundListOps = stringRedisTemplate.boundListOps(key);
        List<GroupDTO> listFromCache = getListFromCache(boundListOps, GroupDTO.class);
        if (CollectionUtils.isNotEmpty(listFromCache)) {
            return listFromCache;
        }
        Message<List<GroupInfoVo>> msg = groupApiService.getGroupInfoList();
        if (msg.isSuccess() && msg.getData() != null) {
            log.info("调用OOP系统查询所有集团信息成功, result: {}", msg.getData());
            List<GroupDTO> list = new ArrayList<>();
            for (GroupInfoVo infoVo : msg.getData()) {
                GroupDTO groupDTO = new GroupDTO();
                BeanUtils.copyProperties(infoVo, groupDTO);
                list.add(groupDTO);
            }
            String[] allGroupArr = list.stream().map(JSONObject::toJSONString).toArray(String[]::new);
            boundListOps.rightPushAll(allGroupArr);
            boundListOps.expire(12, TimeUnit.HOURS);
            return list;
        }
        return null;
    }

    private <E> List<E> getListFromCache(BoundListOperations<String, String> queue, Class<E> clazz) {
        try {
            final long size = Optional.ofNullable(queue.size()).orElse(0L);
            if (size > 0) {
                final List<E> dtos = new ArrayList<>();
                for (long i = 0; i < size; i++) {
                    final String json = Objects.requireNonNull(queue.index(i));
                    dtos.add(JSONObject.parseObject(json, clazz));
                }
                return dtos;
            }
        } catch (Exception e) {
            log.error("从缓存获取信息失败", e);
        }
        return null;
    }

    private String generateKey(final OopTypeEnum typeEnum, final Long id) {
        Assert.notNull(id, "id cannot be null");

        return String.format("%s:%s:%s", getKeyPrefix(), typeEnum.name(), id);
    }


    enum OopTypeEnum {
        /**
         * oop 缓存类型枚举
         */
        SHOP,
        DEALER,
        GROUP,
        BRAND,
        SERIES,
        SPEC,
        REGION
    }
}
