package cn.fw.morax.server.consumer;


import cn.fw.ehr.sdk.api.mq.StaffPostChangeEvent;
import cn.fw.morax.common.utils.MessageFormatUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.morax.service.biz.kpi.KpiPoolBizService;
import cn.fw.morax.service.data.kpi.KpiGroupService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;

/**
 * @author : jiangchao
 * Date: 2022/05/07
 * Time: 17:34
 * Description: 人事调岗
 */
@Slf4j
@Service
@RocketMQMessageListener(topic = StaffPostChangeEvent.TOPIC, consumerGroup = "${spring.application.name}_" + StaffPostChangeEvent.TOPIC)
@RequiredArgsConstructor
public class EhrStaffPostConsumer implements RocketMQListener<StaffPostChangeEvent> {

    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final KpiPoolBizService kpiPoolBizService;
    private final KpiGroupService kpiGroupService;

    /***
     * 人员调岗完成事件
     *
     * 1. 原绩效组为空, 无新绩效组 不算薪酬组变化
     * 2. 原绩效组为空, 有新绩效组 不算薪酬组变化
     * 3. 有原绩效组, 无新绩效组  不算薪酬组变化
     * 4. 有原绩效组, 有新绩效组
     *  1）一致    不算薪酬组变化
     *  2）不致    算薪酬组变化
     * @param result
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void onMessage(StaffPostChangeEvent result) {
        log.info("人事系统人员调岗MQ消息：{}", JSONObject.toJSONString(result));
        //人员调岗完成事件
        KpiGroup originKpiGroup = kpiGroupService.getOne(Wrappers.<KpiGroup>lambdaQuery()
                .in(KpiGroup::getPostId, result.getPostId())
                .in(KpiGroup::getStatus, SettingStatusEnum.EFFECTIVE)
                .apply(MessageFormatUtil.transferWithQuotationMarks("FIND_IN_SET({0}, shop_ids)", result.getShopId()))
        );

        KpiGroup newKpiGroup = kpiGroupService.getOne(Wrappers.<KpiGroup>lambdaQuery()
                .in(KpiGroup::getPostId, result.getNewPostId())
                .in(KpiGroup::getStatus, SettingStatusEnum.EFFECTIVE)
                .apply(MessageFormatUtil.transferWithQuotationMarks("FIND_IN_SET({0}, shop_ids)", result.getNewShopId()))
        );

        //没有新的绩效组，不处理（排除1,3场景）
        if (PublicUtil.isEmpty(newKpiGroup)) {
            log.info("人员调岗，目前没有匹配的绩效组，人员信息：{}", JSONObject.toJSONString(result));
            return;
        }

        //原来绩效组为空（排除2场景）
        if (PublicUtil.isEmpty(originKpiGroup)) {
            log.info("人员调岗，之前没有在绩效组中，人员信息：{}", JSONObject.toJSONString(result));
            return;
        }

        //原绩效组 和 当前绩效组是同一个，不处理（排除4场景第一场景）
        if (originKpiGroup.getId().equals(newKpiGroup.getId())) {
            log.info("人员调岗，原来绩效组和当前绩效组是同一个，人员信息：{}", JSONObject.toJSONString(result));
            return;
        }

        Long userId = result.getStaffId();
        //开始计算绩效时间
        LocalDate startKpiDate = kpiGroupUserBizService.calcStartKpiDate(userId);
        //失效当前所有绩效池数据
        kpiPoolBizService.invalidKpiPool(originKpiGroup, userId, startKpiDate);
    }



}


