package cn.fw.morax.server.controller.erp;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.IndicatorAbleDTO;
import cn.fw.morax.domain.dto.IndicatorsDTO;
import cn.fw.morax.domain.dto.KpiGroupDTO;
import cn.fw.morax.domain.dto.KpiStarRuleDTO;
import cn.fw.morax.domain.dto.query.IndicatorQueryDTO;
import cn.fw.morax.domain.dto.query.KpiGroupQueryDTO;
import cn.fw.morax.domain.dto.query.KpiGroupStaffQueryDTO;
import cn.fw.morax.domain.dto.query.KpiGroupUserPageDTO;
import cn.fw.morax.domain.vo.kpi.*;
import cn.fw.morax.service.biz.kpi.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 绩效组配置
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@RequestMapping("/erp/kpi")
@IgnoreAuth
public class KpiConfigController {

    private final KpiGroupIndicatorBizService kpiGroupIndicatorBizService;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final KpiStarRuleBizService kpiStarRuleBizService;
    private final IndicatorBizService indicatorBizService;
    private final KpiGroupBizService kpiGroupBizService;

    /**
     * 绩效组列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/group-config/page")
    @ControllerMethod("绩效组列表")
    public Message<AppPage<KpiGroupVO>> kpiGroupPage(@Validated KpiGroupQueryDTO dto) {
        return success(kpiGroupBizService.kpiGroupPage(dto));
    }

    /**
     * 绩效组详情
     *
     * @param id 绩效组id
     * @return
     */
    @GetMapping("/group-config")
    @ControllerMethod("绩效组详情")
    public Message<KpiGroupVO> kpiGroupDetail(@NotNull(message = "绩效组id不能为空") Long id) {
        return success(kpiGroupBizService.kpiGroupDetail(id));
    }

    /**
     * 绩效组保存
     *
     * @param dto
     * @return
     */
    @PostMapping("/group-config")
    @ControllerMethod("绩效组保存")
    public Message<KpiGroupSavePromptVO> saveKpi(@Validated @RequestBody KpiGroupDTO dto, @CurrentUser LoginAuthBean currentUser) {
        return success(kpiGroupBizService.saveKpi(dto, currentUser));
    }

    /**
     * 指标详情
     *
     * @param id 绩效组指标id
     * @return
     */
    @GetMapping("/group/indicator")
    @ControllerMethod("指标详情")
    public Message<KpiGroupIndicatorVO> kpiGroupIndicatorDetail(@NotNull(message = "绩效组指标id不能为空") Long id) {
        return success(kpiGroupIndicatorBizService.kpiGroupIndicatorDetail(id));
    }

    /**
     * 指标库列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/indicators/page")
    @ControllerMethod("指标库列表")
    public Message<AppPage<IndicatorsVO>> indicatorsPage(@Validated IndicatorQueryDTO dto) {
        return success(indicatorBizService.indicatorsPage(dto));
    }

    /**
     * 指标库保存
     *
     * @param dto
     * @return
     */
    @PostMapping("/indicators")
    @ControllerMethod("指标库保存")
    public Message<Void> saveIndicator(@Validated @RequestBody IndicatorsDTO dto) {
        indicatorBizService.saveIndicator(dto);
        return success();
    }

    /**
     * 指标库禁用启用
     *
     * @return
     */
    @PutMapping("/indicator/enable")
    @ControllerMethod("指标库禁用启用")
    public Message<Void> enableIndicator(@Validated @RequestBody IndicatorAbleDTO dto) {
        indicatorBizService.enableIndicator(dto.getIndicatorId(), dto.getEnable());
        return success();
    }

    /**
     * 指标详情
     *
     * @param id 指标id
     * @return
     */
    @GetMapping("/indicator")
    @ControllerMethod("指标详情")
    public Message<IndicatorsVO> indicatorDetail(@NotNull(message = "指标id不能为空") Long id) {
        return success(indicatorBizService.indicatorDetail(id));
    }

    /**
     * 获取岗位下的绩效组指标
     *
     * @param postId 岗位id
     * @param shopIds 适用门店不能为空
     * @return
     */
    @GetMapping("/post/indicator")
    @ControllerMethod("获取岗位下的绩效组指标")
    public Message<List<IndicatorsVO>> getPostIndicators(@NotNull(message = "岗位id不能为空") Long postId,
                                                         @NotBlank(message = "适用门店不能为空") String shopIds) {
        return success(indicatorBizService.getPostIndicators(postId, shopIds));
    }

    /**
     * 星级规则查询
     *
     * @param currentUser
     * @return
     */
    @GetMapping("/star-config")
    @ControllerMethod("星级规则查询")
    public Message<KpiStarRuleVO> getKpiStarConfig(@CurrentUser LoginAuthBean currentUser) {
        return success(kpiStarRuleBizService.getKpiStarConfigByGroup(currentUser.getGroupId()));
    }

    /**
     * 绩效星级规则保存
     *
     * @param dto
     * @return
     */
    @PostMapping("/star-config")
    @ControllerMethod("绩效星级规则保存")
    public Message<Void> saveKpiStarConfig(@Validated @RequestBody KpiStarRuleDTO dto) {
        kpiStarRuleBizService.saveKpiStarConfig(dto);
        return success();
    }

    /**
     * 实时查询绩效组人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/real-time/staffs")
    @ControllerMethod("实时查询绩效组人员")
    public Message<List<KpiGroupUserVO>> queryShopPostRealTimeStaffs(@Validated KpiGroupStaffQueryDTO dto) {
        return success(kpiGroupUserBizService.queryShopPostRealTimeStaffs(dto));
    }

    /**
     * 查询绩效组人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/group-config/staffs")
    @ControllerMethod("查询绩效组人员")
    public Message<AppPage<KpiGroupUserVO>> queryShopPostConfigStaffs(@Validated KpiGroupUserPageDTO dto) {
        return success(kpiGroupUserBizService.queryShopPostConfigStaffs(dto));
    }


}
