package cn.fw.morax.server.task;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.morax.common.constant.TimeTaskConstant;
import cn.fw.morax.service.biz.kpi.KpiCalculateService;
import cn.fw.morax.service.biz.kpi.KpiPoolBizService;
import cn.fw.morax.service.biz.kpi.KpiReportService;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.util.concurrent.locks.Lock;

/**
 * @author : kurisu
 * @className : KpiCalcTask
 * @description : 绩效处理任务
 * @date : 2022-04-07 15:29
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class KpiCalcTask {
    private final KpiCalculateService kpiCalculateService;
    private final KpiPoolBizService kpiPoolBizService;
    private final KpiReportService kpiReportService;
    private final DistributedLocker distributedLocker;

    @Autowired
    public KpiCalcTask(final KpiCalculateService kpiCalculateService,
                       final KpiPoolBizService kpiPoolBizService,
                       final KpiReportService kpiReportService,
                       final DistributedLocker distributedLocker) {
        this.kpiCalculateService = kpiCalculateService;
        this.kpiPoolBizService = kpiPoolBizService;
        this.kpiReportService = kpiReportService;
        this.distributedLocker = distributedLocker;
    }


    /**
     * 发送业务上报mq通知
     */
    @Scheduled(cron = TimeTaskConstant.KPI_GROUP_REPORT_NOTICE)
    public void sendNotice() {
        final String lockKey = ":kpiSendNotice";
        Lock lock = distributedLocker.lock(lockKey);
        if (((RLock) lock).isLocked()) {
            kpiReportService.sendNotice();
            lock.unlock();
        }
    }

    /**
     * 持久化业务系统上报的数据
     */
    @Scheduled(initialDelay = 1000 * 10, fixedRate = 1000 * 60)
    public void persistence() {
        kpiReportService.persistenceUserKpiData();
    }


    /**
     * 需要计算绩效得分的绩效组写入缓存 每天凌晨1点10分
     */
    @Scheduled(cron = TimeTaskConstant.CACHE_CALCULATE_KPI_GROUP)
    public void cacheCalculableKpiGroup() {
        kpiCalculateService.cacheCalculableKpiGroupIds(LocalDate.now().minusDays(1L));
    }

    /**
     * 计算绩效得分 每90秒执行一次
     */
    @Scheduled(initialDelay = 1000L * 30, fixedRate = 1000L * 90)
    public void calculateKpi() {
        kpiCalculateService.calculateKpi();
    }

    /**
     * 计算绩效得分率 每120秒执行一次
     */
    @Scheduled(initialDelay = 1000L * 30, fixedRate = 1000L * 120)
    @Async
    public void calcuScoringRate() {
        kpiPoolBizService.calcuScoringRate();
    }

    /**
     * 需要计算绩效排名的绩效组写入缓存 每天凌晨2点
     */
    @Scheduled(cron = TimeTaskConstant.CACHE_CALCULATE_KPI_GROUP_RANK)
    public void cacheCalculableKpiGroupRank() {
        kpiPoolBizService.cacheCalculableKpiGroupRank(LocalDate.now().minusDays(1L));
    }

    /**
     * 计算用户星级 每150秒执行一次
     */
    @Scheduled(initialDelay = 1000L * 30, fixedRate = 1000L * 150)
    @Async
    public void calcuStarLevel() {
        kpiPoolBizService.calcuStarLevel();
    }
}
