package cn.fw.morax.service.biz.calculator.salary;

import cn.fw.common.exception.BusinessException;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.enums.ExtraSalaryTypeEnum;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import cn.fw.morax.domain.enums.SalaryValueTypeEnum;
import cn.fw.morax.service.biz.calculator.Calculator;
import cn.fw.morax.service.biz.salary.SalaryPoolCommonService;
import cn.fw.morax.service.data.salary.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.lang.NonNull;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryBaseCalculator
 * @description : 基础薪酬计算器
 * @date : 2022-04-18 11:29
 */
@Slf4j
public abstract class SalaryBaseCalculator implements Calculator<SalaryGroupProject, SalaryGroupUser, BigDecimal> {
    @Autowired
    protected SalaryGroupProjectSettinService salaryGroupProjectSettinService;
    @Autowired
    protected SalaryRawDataService salaryRawDataService;
    @Autowired
    protected SalaryGroupUserService salaryGroupUserService;
    @Autowired
    protected SalaryProjectHitLogService salaryProjectHitLogService;
    @Autowired
    protected SalaryPoolService salaryPoolService;
    @Autowired
    protected SalaryPoolCommonService salaryPoolCommonService;

    @Value("${salary-project-code.work-age}")
    @Getter
    private String workAgeCode;

    /**
     * 获取计算类型
     *
     * @return 计算类型
     */
    public abstract SalaryCalMethodEnum getCalMethod();

    /**
     * 计算动态类型的金额
     *
     * @param pool                薪酬池
     * @param date                日期
     * @param extraSalaryTypeEnum 动态薪酬类型
     * @return 薪酬池详情
     */
    public SalaryPoolDetail calcDynamicMoney(@NonNull SalaryPool pool, @NonNull LocalDate date, @NonNull ExtraSalaryTypeEnum extraSalaryTypeEnum) {
        return null;
    }

    /**
     * 查询薪酬指标详细配置
     *
     * @param salaryGroupProjectId 薪资组指标配置id
     * @return
     */
    protected List<SalaryGroupProjectSettin> queryProjectSettins(final Long salaryGroupProjectId) {
        List<SalaryGroupProjectSettin> list = salaryGroupProjectSettinService.list(Wrappers.<SalaryGroupProjectSettin>lambdaQuery()
                .eq(SalaryGroupProjectSettin::getSalaryGroupProjectId, salaryGroupProjectId)
                .eq(SalaryGroupProjectSettin::getYn, Boolean.TRUE));
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list;
    }


    /**
     * 查询薪酬指标的基础数据
     *
     * @param userId
     * @param date
     * @param projectCode
     * @param valueType
     * @return
     */
    protected Optional<SalaryRawData> queryRawData(Long userId, LocalDate date, String projectCode, SalaryValueTypeEnum valueType) {
        return Optional.ofNullable(salaryRawDataService.getOne(Wrappers.<SalaryRawData>lambdaQuery()
                        .eq(SalaryRawData::getUserId, userId)
                        .eq(SalaryRawData::getValueType, valueType)
                        .eq(SalaryRawData::getDataDate, date)
                        .eq(SalaryRawData::getSalaryProjectCode, projectCode)
                        .eq(SalaryRawData::getYn, Boolean.TRUE)
                , Boolean.FALSE));
    }

    /**
     * 查询薪酬指标的基础数据
     *
     * @param userId
     * @param date
     * @param projectCode
     * @param valueType
     * @return
     */
    protected Optional<JSONObject> queryRawDataValue(Long userId, LocalDate date, String projectCode, SalaryValueTypeEnum valueType) {
        Optional<SalaryRawData> rawData = queryRawData(userId, date, projectCode, valueType);
        return checkRawDataAndGetDataValue(rawData, userId, projectCode);
    }


    /**
     * 查询工作天数占比
     *
     * @param user
     * @return
     */
    protected BigDecimal calcWorkDayProportion(SalaryGroupUser user) {
        int count = salaryGroupUserService.count(Wrappers.<SalaryGroupUser>lambdaQuery()
                .eq(SalaryGroupUser::getUserId, user.getUserId())
                .eq(SalaryGroupUser::getPostId, user.getPostId())
                .eq(SalaryGroupUser::getShopId, user.getShopId())
                .eq(SalaryGroupUser::getProbationer, Boolean.FALSE)
                .eq(SalaryGroupUser::getFrozen, Boolean.FALSE)
                .eq(SalaryGroupUser::getYn, Boolean.TRUE)
                .le(SalaryGroupUser::getDataDate, user.getDataDate())
        );
        if (count <= 0) {
            return BigDecimal.ZERO;
        }

        int lengthOfMonth = YearMonth.from(user.getDataDate()).lengthOfMonth();
        return new BigDecimal(count).divide(new BigDecimal(lengthOfMonth), 2, RoundingMode.HALF_UP);
    }

    /**
     * 校验数据
     *
     * @param rawData
     */
    protected Optional<JSONObject> checkRawDataAndGetDataValue(Optional<SalaryRawData> rawData, Long userId, String salaryProjectCode) {
        JSONObject projectValue = (rawData.isPresent()) ? JSONObject.parseObject(rawData.get().getSalaryProjectValue()) : null;
        return Optional.ofNullable(projectValue);

//        BV.notNull(rawData, () -> MessageFormatUtil.MessageFormatTransfer("用户[{0}]的[{1}]数据不存在", userId, salaryProjectCode));
//        String salaryProjectValue = rawData.getSalaryProjectValue();
//        JSONObject projectValue = JSONObject.parseObject(salaryProjectValue);
//        BV.notNull(projectValue, () -> MessageFormatUtil.MessageFormatTransfer("[{0}]计算失败", salaryProjectCode));
//        return projectValue;
    }

    /**
     * 记录命中配置
     *
     * @param projectId
     * @param settinId
     * @param salaryValue
     * @param user
     */
    protected void saveHitLog(Long projectId, Long settinId, BigDecimal salaryValue, SalaryGroupUser user) {
        SalaryPool salaryPool = querySalaryPool(user);
        SalaryProjectHitLog hitLog = SalaryProjectHitLog.builder()
                .salaryProjectId(projectId)
                .salaryPoolId(salaryPool.getId())
                .salaryGroupProjectSettinId(settinId)
                .salaryValue(salaryValue)
                .dataDate(user.getDataDate())
                .groupId(user.getGroupId())
                .build();
        salaryProjectHitLogService.saveUnique(hitLog);
    }

    /**
     * 查询薪酬池
     *
     * @param user
     * @return
     */
    protected SalaryPool querySalaryPool(SalaryGroupUser user) {
        SalaryPool salaryPool = salaryPoolCommonService.inspectionPool(user);
        if (PublicUtil.isEmpty(salaryPool)) {
            log.error("薪酬池数据不存在，userId：{}，postId:{}，salaryGroupId：{}", user.getUserId(), user.getPostId(), user.getSalaryGroupId());
            throw new BusinessException("薪酬池数据不存在");
        }
        return salaryPool;
    }
}
