package cn.fw.morax.service.biz.kpi;

import cn.fw.common.exception.BusinessException;
import cn.fw.morax.common.utils.MessageFormatUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroupUser;
import cn.fw.morax.domain.db.kpi.KpiPool;
import cn.fw.morax.domain.db.salary.SalaryGroupUser;
import cn.fw.morax.domain.db.salary.SalaryPool;
import cn.fw.morax.domain.enums.StarLevelEnum;
import cn.fw.morax.service.data.kpi.KpiPoolService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.YearMonth;
import java.util.Objects;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : KpiPoolCommonService
 * @description : 绩效池公用方法
 * @date : 2022-09-27 11:40
 */
@Slf4j
@Service
@AllArgsConstructor
public class KpiPoolCommonService {

    private KpiPoolService kpiPoolService;

    /**
     * 查询绩效池
     *
     * @param user
     * @return
     */
    public KpiPool queryPool(SalaryGroupUser user) {
        KpiPool kpiPool = kpiPoolService.getOne(Wrappers.<KpiPool>lambdaQuery()
                        .eq(KpiPool::getUserId, user.getUserId())
                        .eq(KpiPool::getPostId, user.getPostId())
                        .eq(KpiPool::getShopId, user.getShopId())
                        .eq(KpiPool::getMonthly, YearMonth.from(user.getDataDate()))
                        .eq(KpiPool::getYn, Boolean.TRUE)
                , Boolean.FALSE);
        if (PublicUtil.isEmpty(kpiPool)) {
            log.error("绩效数据不存在，用户：{}", JSON.toJSONString(user));
            throw new BusinessException("用户[" + user.getUserId() + "]的绩效数据不存在");
        }
        return kpiPool;
    }

    /**
     * 查询绩效池
     *
     * @param pool
     * @return
     */
    public KpiPool queryPool(SalaryPool pool) {
        KpiPool kpiPool = kpiPoolService.getOne(Wrappers.<KpiPool>lambdaQuery()
                        .eq(KpiPool::getUserId, pool.getUserId())
                        .eq(KpiPool::getPostId, pool.getPostId())
                        .eq(KpiPool::getShopId, pool.getShopId())
                        .eq(KpiPool::getMonthly, pool.getMonthly())
                        .eq(KpiPool::getYn, Boolean.TRUE)
                , Boolean.FALSE);
        if (PublicUtil.isEmpty(kpiPool)) {
            log.error("绩效数据不存在，薪酬池：{}", JSON.toJSONString(pool));
            throw new BusinessException("用户[" + pool.getUserId() + "]的绩效数据不存在");
        }
        return kpiPool;
    }

    /**
     * 查询绩效池（上报绩效、计算绩效得分用）
     *
     * @param user
     * @return
     */
    public KpiPool inspectionPool(KpiGroupUser user) {
        KpiPool kpiPool = kpiPoolService.getOne(Wrappers.<KpiPool>lambdaQuery()
                        .eq(KpiPool::getKgc, user.getKgc())
                        .eq(KpiPool::getUserId, user.getUserId())
                        .eq(KpiPool::getMonthly, YearMonth.from(user.getDataDate()))
                , Boolean.FALSE);

        if (Objects.isNull(kpiPool)) {
            kpiPool = kpiPoolService.getOne(Wrappers.<KpiPool>lambdaQuery()
                            .eq(KpiPool::getPostId, user.getPostId())
                            .eq(KpiPool::getShopId, user.getShopId())
                            .eq(KpiPool::getUserId, user.getUserId())
                            .eq(KpiPool::getMonthly, YearMonth.from(user.getDataDate()))
                    , Boolean.FALSE);

            if (Objects.isNull(kpiPool)) {
                kpiPool = createPoolData(user);
            } else {
                kpiPool = this.modifyPoolData(kpiPool, user);
            }
        } else if ((! kpiPool.getShopId().equals(user.getShopId())) ||
                   (! kpiPool.getKpiGroupId().equals(user.getKpiGroupId())) ){
            //若门店信息与员工在职门店不匹配，说明员工是调门店，绩效组没变
            //若绩效组id不匹配，说明绩效组编辑生成了新绩效组，对应绩效池中的绩效组id修改
            kpiPool = this.modifyPoolData(kpiPool, user);
        }
        return kpiPool;
    }

    @Transactional(rollbackFor = Exception.class)
    public KpiPool modifyPoolData(KpiPool pool, KpiGroupUser user) {
        pool.setKpiGroupId(user.getKpiGroupId());
        pool.setShopId(user.getShopId());
        pool.setShopName(user.getShopName());
        pool.setKgc(user.getKgc());
        kpiPoolService.updateById(pool);
        return pool;
    }

    @Transactional(rollbackFor = Exception.class)
    public KpiPool createPoolData(KpiGroupUser groupUser) {
        KpiPool pool = new KpiPool();
        pool.setKpiGroupId(groupUser.getKpiGroupId());
        pool.setKgc(groupUser.getKgc());
        pool.setUserId(groupUser.getUserId());
        pool.setUserName(groupUser.getUserName());
        pool.setPostId(groupUser.getPostId());
        pool.setPostName(groupUser.getPostName());
        pool.setShopId(groupUser.getShopId());
        pool.setShopName(groupUser.getShopName());
        pool.setStarLevel(StarLevelEnum.C);
        pool.setActualStar(StarLevelEnum.C);
        pool.setRevoked(Boolean.FALSE);
        pool.setInclusion(!Boolean.TRUE.equals(groupUser.getIgnored()));
        pool.setRegular(Boolean.FALSE);
        pool.setMonthly(YearMonth.from(groupUser.getDataDate()));
        pool.setGroupId(groupUser.getGroupId());
        pool.setYn(Boolean.TRUE);
        pool.setKpiScore(BigDecimal.ZERO);
        pool.setKpiScoreRatio(BigDecimal.ZERO);
        kpiPoolService.save(pool);
        return pool;
    }

}
